// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:flutter/services.dart' show ServicesBinding, rootBundle;
import 'package:wyatt_i18n/src/core/enums/format.dart';
import 'package:wyatt_i18n/src/core/exceptions/exceptions.dart';
import 'package:wyatt_i18n/src/domain/data_sources/i18n_data_source.dart';

class AssetsFileDataSourceImpl extends I18nDataSource {
  const AssetsFileDataSourceImpl({super.format = Format.arb}) : super();

  /// Checks if the given [assetPath] is a local asset.
  Future<bool> assetExists(String assetPath) async {
    final encoded = utf8.encoder.convert(
      Uri(path: Uri.encodeFull(assetPath)).path,
    );
    final asset = await ServicesBinding.instance.defaultBinaryMessenger
        .send('flutter/assets', encoded.buffer.asByteData());

    return asset != null;
  }

  Future<String?> _tryLoad(String basePath) async {
    String? content;
    try {
      for (final ext in super.format.extensions) {
        if (await assetExists('$basePath.$ext')) {
          content = await rootBundle.loadString('$basePath.$ext');
        }
      }
    } catch (_) {
      throw SourceNotFoundException(basePath, format: format);
    }

    if (content == null) {
      throw SourceNotFoundException(basePath, format: format);
    }

    return content;
  }

  Future<String> _load(String locale) async {
    String? content;

    try {
      content = await _tryLoad('assets/i18n.$locale');
    } catch (_) {
      content = await _tryLoad('assets/i18n');
    }

    /// Content can't be null at this point.
    /// Because if it is, previous [_tryLoad] calls would have t
    /// hrown an exception.
    return content!;
  }

  /// Loads the i18n file from Assets folder.
  ///
  /// The i18n file must be named `i18n.<locale>.<extension>` and must
  /// be specified in the `pubspec.yaml` file.
  @override
  Future<String> load(String locale) async {
    final content = await _load(locale);

    return content;
  }
}
